<?php
/**
 * Plugin Name: Emails Catch All
 * Plugin URI: https://iuliacazan.ro/emails-catch-all/
 * Description: This is a plugin that allows you to set a <strong>catch all emails address</strong> that will <strong>receive a copy of all the emails sent</strong> from the application or that will <strong>replace the recipients for all the emails sent</strong>. Also, the plugin allows to set the content type of the emails and to record the history of the outgoing messages, set SMTP authentication and exposes errors on failed test.
 * Text Domain: secas
 * Domain Path: /langs
 * Version: 2.2
 * Author: Iulia Cazan
 * Author URI: https://profiles.wordpress.org/iulia-cazan
 * Donate link: https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=JJA37EHZXWUTJ
 * License: GPL2
 *
 * @package ic-devops
 *
 * Copyright (C) 2016-2018 Iulia Cazan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

// The current plugin version of BD.
define( 'SISANU_EMAILS_CATCH_ALL_DB_VERSION', 2.2 );

// The current plugin custom table name.
define( 'SISANU_EMAILS_CATCH_ALL_TABLE', 'emails_catch_all' );

/**
 * Class for Emails Catch All.
 */
class SISANU_Emails_Catch_All {
	/**
	 * Class instance.
	 *
	 * @var object
	 */
	private static $instance;

	/**
	 * Class settings.
	 *
	 * @var object
	 */
	public static $settings;

	/**
	 * Number of records per page.
	 *
	 * @var integer
	 */
	public static $records_per_page = 10;

	/**
	 * URL to the plugin.
	 *
	 * @var string
	 */
	public static $plugin_url = 'options-general.php?page=emails-catch-all-settings';

	/**
	 * The plugin option name.
	 *
	 * @var string
	 */
	public static $option_name = 'emails_catch_all_settings';

	/**
	 * The plugin option prefix.
	 *
	 * @var string
	 */
	public static $option_prefix = 'SECAS_';

	/**
	 * Get active object instance.
	 *
	 * @access public
	 * @static
	 *
	 * @return object
	 */
	public static function get_instance() {
		if ( ! self::$instance ) {
			self::$instance = new SISANU_Emails_Catch_All();
		}
		return self::$instance;
	}

	/**
	 * Class constructor.
	 *
	 * @return void
	 */
	public function __construct() {
		$this->init();
	}

	/**
	 * Initialize the plugin.
	 *
	 * @access private
	 *
	 * @return void
	 */
	private function init() {
		$ob_class = get_called_class();

		self::get_options();
		add_action( 'plugins_loaded', array( $ob_class, 'load_textdomain' ) );

		if ( ! empty( self::$settings->email ) || ! empty( self::$settings->just_record ) ) {
			// Enable the plugin filters only if this is necessary.
			add_filter( 'wp_mail', array( $ob_class, 'wp_mail_catch_all' ), 99 );
			add_filter( 'wp_mail_content_type', array( $ob_class, 'set_content_type' ), 99 );

			// Maybe alter the mailer settings.
			add_action( 'phpmailer_init', array( $ob_class, 'maybe_phpmailer_settings' ), 99 );
		}
		if ( is_admin() ) {
			add_action( 'init', array( $ob_class, 'update_settings' ) );
			add_action( 'admin_menu', array( $ob_class, 'admin_menu' ) );
			add_action( 'admin_enqueue_scripts', array( $ob_class, 'load_assets' ) );
			add_action( 'wp_ajax_secas_navigate_to_page', array( $ob_class, 'load_emails_page' ) );

			$action_links = 'plugin_action_links_' . plugin_basename( __FILE__ );
			add_filter( $action_links, array( $ob_class, 'plugin_action_links' ) );
		}
	}

	/**
	 * Maybe update the phpmailer settings.
	 *
	 * @param object $phpmailer PHP Mailer instance.
	 * @return void
	 */
	public static function maybe_phpmailer_settings( $phpmailer ) {
		if ( ! empty( self::$settings->smtp_auth ) ) {
			$phpmailer->isSMTP();
			$phpmailer->SMTPAuth = self::$settings->smtp_auth;
		}
		if ( ! empty( self::$settings->smtp_host ) ) {
			$phpmailer->Host = self::$settings->smtp_host;
		}
		if ( ! empty( self::$settings->smtp_port ) ) {
			$phpmailer->Port = self::$settings->smtp_port;
		}
		if ( ! empty( self::$settings->smtp_uname ) ) {
			$phpmailer->Username = self::$settings->smtp_uname;
		}
		if ( ! empty( self::$settings->smtp_upass ) ) {
			$phpmailer->Password = self::$settings->smtp_upass;
		}
		if ( ! empty( self::$settings->smtp_secure ) ) {
			$phpmailer->SMTPSecure = self::$settings->smtp_secure;
		}
		if ( ! empty( self::$settings->smtp_from ) ) {
			$phpmailer->From = self::$settings->smtp_from;
		}
		if ( ! empty( self::$settings->smtp_from_name ) ) {
			$phpmailer->FromName = self::$settings->smtp_from_name;
		}
	}

	/**
	 * Load text domain for internalization
	 *
	 * @access public
	 * @static
	 *
	 * @return void
	 */
	function load_textdomain() {
		load_plugin_textdomain( 'secas', false, basename( dirname( __FILE__ ) ) . '/langs/' );
	}

	/**
	 * Return the specified email content type.
	 *
	 * @access public
	 * @static
	 *
	 * @return string
	 */
	public static function set_content_type() {
		if ( 'html' === self::$settings->content_type ) {
			return 'text/html';
		} else {
			return 'text/plain';
		}
	}

	/**
	 * Get the plugin settings.
	 *
	 * @access public
	 * @static
	 *
	 * @return void
	 */
	public static function get_options() {
		self::$settings = (object) array(
			'email'          => '',
			'just_record'    => false,
			'recipient'      => 'receive',
			'history'        => false,
			'content_type'   => 'html',
			'smtp_auth'      => false,
			'smtp_host'      => '',
			'smtp_port'      => 25,
			'smtp_uname'     => '',
			'smtp_upass'     => '',
			'smtp_secure'    => 'tls',
			'smtp_from'      => '',
			'smtp_from_name' => '',
		);

		$all_opt = get_option( self::$option_name );
		if ( ! empty( $all_opt ) ) {
			foreach ( $all_opt as $key => $value ) {
				if ( 'history' === $key || 'just_record' === $key || 'smtp_auth' === $key ) {
					$value = (boolean) $value;
				}
				self::$settings->{$key} = $value;
			}
		}
	}

	/**
	 * Add the plugin menu
	 *
	 * @access public
	 * @static
	 *
	 * @return void
	 */
	public static function admin_menu() {
		add_submenu_page(
			'options-general.php',
			__( 'Emails Catch All', 'secas' ),
			'<div class="dashicons dashicons-email ic-devops"></div> ' . __( 'Emails Catch All', 'secas' ),
			'manage_options',
			'emails-catch-all-settings',
			array( get_called_class(), 'emails_catch_all_settings' )
		);
	}

	/**
	 * Load scripts and stypes used by the plugin.
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	public static function load_assets() {
		wp_register_script(
			'secas-custom',
			plugins_url( '/assets/custom.js', plugin_basename( __FILE__ ) ),
			array( 'jquery' ),
			filemtime( plugin_dir_path( __FILE__ ) . 'assets/custom.js' )
		);
		wp_localize_script(
			'secas-custom',
			'secasSettings',
			array(
				'ajaxUrl'           => admin_url( 'admin-ajax.php' ),
				'cleanupConfirmAll' => __( 'Are you sure you want to delete all the records?', 'secas' ),
				'cleanupConfirmOne' => __( 'Are you sure you want to delete this record?', 'secas' ),
			)
		);
		wp_enqueue_script( 'secas-custom' );

		wp_enqueue_style(
			'secas-custom',
			plugins_url( '/assets/custom.css', plugin_basename( __FILE__ ) ),
			array(),
			filemtime( plugin_dir_path( __FILE__ ) . 'assets/custom.css' )
		);
	}

	/**
	 * Maybe update the plugin settings and send a test email.
	 *
	 * @return void
	 */
	public static function update_settings() {
		// User can save the options.
		$used_nonce = filter_input( INPUT_POST, '_secas_settings_nonce', FILTER_DEFAULT );
		if ( ! empty( $used_nonce ) && wp_verify_nonce( $used_nonce, '_secas_settings_save' ) ) {
			if ( ! current_user_can( 'manage_options' ) ) {
				wp_die( esc_html( 'Action not allowed.', 'secas' ) );
			}

			$all_opt = self::$settings;
			$posted  = filter_input( INPUT_POST, '_secas_settings', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
			foreach ( $all_opt as $key => $value ) {
				$val = ( isset( $posted[ $key ] ) ) ? sanitize_text_field( $posted[ $key ] ) : '';
				if ( 'history' === $key || 'just_record' === $key || 'smtp_auth' === $key ) {
					$val = (boolean) $val;
				}
				$all_opt->{$key} = trim( $val );
			}
			update_option( self::$option_name, $all_opt );
			if ( isset( $posted['test'] ) ) {
				self::get_options();
				self::send_test_email();
			}
		}
	}

	/**
	 * Send a test email.
	 *
	 * @access public
	 * @static
	 *
	 * @return void|object
	 */
	public static function send_test_email() {
		self::get_options();

		if ( is_multisite() ) {
			$blogname = $GLOBALS['current_site']->site_name;
		} else {
			$blogname = wp_specialchars_decode( get_option( 'blogname' ), ENT_QUOTES );
		}
		// Translators: %s site name.
		$title   = sprintf( __( '[%s] Test Email Catch All', 'secas' ), $blogname );
		$message = sprintf(
			// Translators: %1$s from %2$s date %3$s to %4$s url.
			__( 'This is <em>a sample test email</em> from <b>%1$s</b>, sent on <u>%2$s</u>, having the initial recipient <b>%3$s</b>.<br />Visit the site at <a href="%4$s">%4$s</a>', 'secas' ),
			$blogname,
			current_time( 'mysql' ),
			get_option( 'admin_email', true ),
			get_site_url(),
			get_site_url()
		);

		$user_email = get_option( 'admin_email' );
		$headers    = array();
		$headers[]  = 'Cc: ' . $user_email;
		$headers[]  = 'Bcc: ' . $user_email;
		$result     = false;
		if ( ! empty( $message ) ) {
			add_action( 'wp_mail_failed', array( get_called_class(), 'on_mail_error' ), 10, 1 );
			$result = wp_mail( $user_email, wp_specialchars_decode( $title ), $message, $headers );
			if ( true === $result ) {
				add_action( 'admin_notices', array( get_called_class(), 'on_mail_success_notice' ) );
			}
		}
		return $result;
	}

	/**
	 * Add the admin error message for failed email test.
	 *
	 * @param object $wp_error WP_Error object.
	 * @return void
	 */
	public static function on_mail_error( $wp_error ) {
		if ( ! empty( $wp_error ) ) {
			global $secas_test_mail_error;
			$secas_test_mail_error = $wp_error;
			add_action( 'admin_notices', array( get_called_class(), 'on_mail_error_notice' ) );
		}
	}

	/**
	 * Output the admin error message for failed email test.
	 *
	 * @return void
	 */
	public static function on_mail_error_notice() {
		global $secas_test_mail_error;
		$class   = 'notice notice-error';
		$message = __( 'An error has occurred when sending the test email: ', 'secas' );
		if ( ! empty( $secas_test_mail_error->errors['wp_mail_failed'] ) ) {
			$message .= implode( ' &bull; ', $secas_test_mail_error->errors['wp_mail_failed'] );
		}

		printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
	}

	/**
	 * Output the admin success message for email test sent.
	 *
	 * @return void
	 */
	public static function on_mail_success_notice() {
		$class   = 'notice notice-success is-dismissible';
		$message = __( 'Success! The test message has been sent.', 'secas' );
		printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
	}

	/**
	 * Return the pagination links.
	 *
	 * @access public
	 * @static
	 *
	 * @param integer $total_pages   Total pages.
	 * @param integer $current_page  Current page.
	 * @param integer $total_records Total records.
	 * @return string
	 */
	public static function history_pagination( $total_pages = 1, $current_page = 1, $total_records = 0 ) {
		$result = '
		<br><table class="wp-list-table" width="100%">
		<tr>';
		$result .= '<td width="20%"><h3>' . esc_html__( 'Recorded Emails', 'secas' ) . '</h3></td>';
		if ( $total_pages > 1 ) {
			$range = 4;
			$start = ceil( $current_page / $range ) * $range - $range + 1;
			$end   = $start + $range - 1;
			if ( $end > $total_pages ) {
				$end = $total_pages;
			}

			$result .= '<td width="20%">' . sprintf(
				// Translators: %1$d - current page, %2$d - total pages.
				__( 'Page %1$d of %2$d', 'secas' ),
				$current_page,
				$total_pages
			) . ' </td>';
			$result .= '<td>';

			if ( $start > $range ) {
				$result .= '<a class="secas-page button" data-page="1" href="' . admin_url( self::$plugin_url . '&cp=1' ) . '">&laquo;</a> ';
			} else {
				$result .= '<a class="button disabled">&laquo;</a> ';
			}

			if ( $current_page > 1 ) {
				$result .= '<a class="secas-page button" data-page="' . ( (int) $current_page - 1 ) . '" href="' . admin_url( self::$plugin_url . '&cp=' . ( (int) $current_page - 1 ) ) . '">&lsaquo;</a> ';
			} else {
				$result .= '<a class="button disabled">&lsaquo;</a> ';
			}

			if ( $start > $range ) {
				$result .= ' ... ';
			}

			for ( $i = $start; $i <= $end; $i ++ ) {
				$class   = ( (int) $i === (int) $current_page ) ? ' button-primary' : '';
				$result .= '<a class="secas-page button' . $class . '" data-page="' . (int) $i . '" href="' . admin_url( self::$plugin_url . '&cp=' . $i ) . '">' . $i . '</a> ';
			}
			if ( $end < $total_pages ) {
				$result .= ' ...  ';
			}

			if ( $current_page < $total_pages ) {
				$result .= '<a class="secas-page button" data-page="' . ( (int) $current_page + 1 ) . '" href="' . admin_url( self::$plugin_url . '&cp=' . ( (int) $current_page + 1 ) ) . '">&rsaquo;</a> ';
			} else {
				$result .= '<a class="button disabled">&rsaquo;</a>';
			}

			if ( $current_page < $total_pages ) {
				$result .= '<a class="secas-page button" data-page="' . $total_pages . '" href="' . admin_url( self::$plugin_url . '&cp=' . $total_pages ) . '">&raquo;</a> ';
			} else {
				$result .= '<a class="button disabled">&raquo;</a>';
			}

			$result .= '</td>';
		}

		$result .= '<td align="right">' . $total_records . ' ' . esc_html__( 'records', 'secas' ) . '</td>';

		if ( ! empty( $total_records ) ) {
			$result .= '<td align="right"><a class="button" href="javascript:void(0);" onclick="secas_cleanup(\'all\', \'' . $current_page . '\')">' . esc_html__( 'Reset the emails history', 'secas' ) . '</a></td>';
		}

		$result .= '</tr></table><br>';
		return $result;
	}

	/**
	 * Load a page of records.
	 *
	 * @return void
	 */
	public static function load_emails_page() {
		global $wpdb;
		$is_ajax = filter_input( INPUT_POST, 'is_ajax', FILTER_DEFAULT );
		$cp      = filter_input( INPUT_POST, 'cp', FILTER_SANITIZE_NUMBER_INT );
		if ( empty( $cp ) ) {
			$cp = filter_input( INPUT_GET, 'cp', FILTER_SANITIZE_NUMBER_INT );
		}

		$cleanup = filter_input( INPUT_POST, 'cleanup', FILTER_DEFAULT );
		if ( ! empty( $cleanup ) ) {
			self::emails_catch_all_cleanup_records( $cleanup );
		}

		$search = filter_input( INPUT_POST, 'search', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
		$cp     = ( ! empty( $cp ) ) ? abs( (int) $cp ) : 1;

		$t_query = 'SELECT count(id) as total FROM ' . SISANU_EMAILS_CATCH_ALL_TABLE . ' WHERE 1=1 ';
		$q_args  = array();
		if ( ! empty( $search['date'] ) ) {
			$t_query  .= ' AND ( `date` >= %d AND `date` <= %d )';
			$q_args[] = strtotime( $search['date'] );
			$q_args[] = strtotime( $search['date'] ) + DAY_IN_SECONDS - 1;
		}
		if ( ! empty( $search['email'] ) ) {
			$t_query .= ' AND ( `all` LIKE %s )';
			$q_args[] = '%' . $wpdb->esc_like( $search['email'] ) . '%';
		}
		if ( ! empty( $search['text'] ) ) {
			$t_query .= ' AND ( `all` LIKE %s )';
			$q_args[] = '%' . $wpdb->esc_like( $search['text'] ) . '%';
		}
		$total = $wpdb->get_results( $wpdb->prepare( $t_query, $q_args ) ); // WPCS: Unprepared SQL OK. - for false positive query placeholders.
		$total = (int) $total[0]->total;
		$perp  = self::$records_per_page;
		$pages = ceil( $total / $perp );
		$pages = ( ! empty( $pages ) ) ? abs( (int) $pages ) : 1;
		if ( $cp > $pages ) {
			$cp = $pages;
		}
		$offset  = ( $cp - 1 ) * $perp;

		$query  = 'SELECT * FROM ' . SISANU_EMAILS_CATCH_ALL_TABLE . ' WHERE 1=1 ';
		$q_args = array();
		if ( ! empty( $search['date'] ) ) {
			$query   .= ' AND ( `date` >= %d AND `date` <= %d )';
			$q_args[] = strtotime( $search['date'] );
			$q_args[] = strtotime( $search['date'] ) + DAY_IN_SECONDS - 1;
		}
		if ( ! empty( $search['email'] ) ) {
			$query   .= ' AND ( `all` LIKE %s )';
			$q_args[] = '%' . $wpdb->esc_like( $search['email'] ) . '%';
		}
		if ( ! empty( $search['text'] ) ) {
			$query   .= ' AND ( `all` LIKE %s )';
			$q_args[] = '%' . $wpdb->esc_like( $search['text'] ) . '%';
		}
		$query   .= ' ORDER BY date DESC LIMIT %d,%d';
		$q_args[] = $offset;
		$q_args[] = $perp;

		$records = $wpdb->get_results( $wpdb->prepare( $query, $q_args ) ); // WPCS: Unprepared SQL OK. - for false positive query placeholders.

		$date_format = get_option( 'date_format' );
		$time_format = get_option( 'time_format' );
		$pagination  = self::history_pagination( $pages, $cp, $total );
		if ( ! empty( $pagination ) ) {
			echo $pagination; // WPCS: XSS OK.
		}
		?>

		<table class="wp-list-table widefat fixed striped">
			<thead>
				<tr>
					<th width="60"><?php esc_html_e( 'No.', 'secas' ); ?></th>
					<th width="140"><?php esc_html_e( 'Date', 'secas' ); ?></th>
					<th width="220"><?php esc_html_e( 'Intended Recipient', 'secas' ); ?></th>
					<th width="220"><?php esc_html_e( 'Final Recipient', 'secas' ); ?></th>
					<th><?php esc_html_e( 'Subject', 'secas' ); ?> / <?php esc_html_e( 'Message', 'secas' ); ?></th>
					<th width="150"> </th>
				</tr>

				<tr id="secas-search-wrap">
					<th colspan="2"><input type="date" name="secas_search[date]" id="secas_search_date" pattern="[0-9]{4}-[0-9]{2}-[0-9]{2}" value="<?php echo esc_attr( $search['date'] ); ?>"></th>
					<th colspan="2"><input type="text" name="secas_search[email]" id="secas_search_email" placeholder="<?php esc_attr_e( 'Intended or final recipient email address', 'secas' ); ?>" value="<?php echo esc_attr( $search['email'] ); ?>"></th>
					<th>
						<input type="text" name="secas_search[text]" id="secas_search_text" placeholder="<?php esc_attr_e( 'Subject or content of the message', 'secas' ); ?>" value="<?php echo esc_attr( $search['text'] ); ?>">
					</th>
					<th align="right">
						<a href="javascript:void(0);" class="button" id="secas_reset_search"><?php esc_html_e( 'Reset Filters', 'secas' ); ?></a>
					</th>
				</tr>
			</thead>
		<?php if ( empty( $records ) ) : ?>
			</table>
			<hr /><?php esc_html_e( 'No recorded history.', 'secas' ); ?>
		<?php else : ?>
				<tbody>
					<?php wp_nonce_field( '_emails_history_rows_save', '_emails_history_rows_nonce' ); ?>
					<?php foreach ( $records as $i => $item ) : ?>
						<?php
						$row = maybe_unserialize( $item->all );

						$initial_receiver = '<b class="label">' . esc_html__( 'TO', 'secas' ) . '</b> ' . str_replace( ',', ', ', $row['initial']['to'] );
						if ( ! empty( $row['initial']['cc'] ) ) {
							$initial_receiver .= '<br><b class="label">' . esc_html__( 'CC', 'secas' ) . '</b> ' . str_replace( 'cc: ', '', str_replace( ',', ', ', strtolower( $row['initial']['cc'] ) ) );
						}
						if ( ! empty( $row['initial']['bcc'] ) ) {
							$initial_receiver .= '<br><b class="label">' . esc_html__( 'BCC', 'secas' ) . '</b> ' . str_replace( 'bcc: ', '', str_replace( ',', ', ', strtolower( $row['initial']['bcc'] ) ) );
						}
						$row['final']['to'] = ( ! empty( $row['final']['bcc'] ) ) ? $row['final']['to'] . ', ' . $row['final']['bcc'] : $row['final']['to'];
						$row['final']['to'] = str_replace( ',', ', ', $row['final']['to'] );

						$counter = ( $cp - 1 ) * (int) self::$records_per_page + $i + 1;
						?>
						<tr id="history_row_<?php echo (int) $item->id; ?>">
							<td><?php echo (int) $counter; ?>.</td>
							<td>
								<?php echo esc_html( date_i18n( $date_format . ' ' . $time_format, $row['date'] ) ); ?>
								<br />
								<a href="javascript:void(0);" class="button" onclick="secas_cleanup('<?php echo (int) $item->id; ?>', '<?php echo (int) $cp; ?>')"><?php esc_html_e( 'Delete', 'secas' ); ?></a>
							</td>
							<td>
								<?php
								echo sprintf(
									'%1$s%2$s%3$s',
									( 'replace' === $row['type'] ) ? '<strike>' : '',
									$initial_receiver,
									( 'replace' === $row['type'] ) ? '</strike>' : ''
								); // WPCS: XSS OK.
								?>
							</td>
							<td>
								<?php esc_html_e( 'Type', 'secas' ); ?>:
								<b><?php echo esc_html( $row['type'] ); ?></b>
								<br><div class="secas-border"><?php echo esc_html( $row['final']['to'] ); ?></div>
							</td>
							<td colspan="2">
								<h3><?php echo esc_html( $row['final']['subject'] ); ?></h3>
								<?php echo ( ! empty( $row['final']['message'] ) ) ? wp_kses_post( wpautop( $row['final']['message'] ) ) : ''; ?>
							</td>
						</tr>
					<?php endforeach; ?>
				</tbody>
			</table>
			<?php
			if ( ! empty( $pagination ) ) {
				echo $pagination; // WPCS: XSS OK.
			}
			?>
		<?php endif; ?>
		<?php
		if ( ! empty( $is_ajax ) ) {
			die();
		}
	}

	/**
	 * Execute the full cleanup of the plugin history.
	 *
	 * @access public
	 * @static
	 *
	 * @return void
	 */
	public static function history_cleanup() {
		global $wpdb;
		$wpdb->query( ' DELETE from ' . SISANU_EMAILS_CATCH_ALL_TABLE );
	}

	/**
	 * Execute the cleanup.
	 *
	 * @access public
	 * @static
	 *
	 * @param string $val The value for the cleanup.
	 * @return void
	 */
	public static function emails_catch_all_cleanup_records( $val ) {
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( esc_html( 'Action not allowed.', 'secas' ) );
		} else {
			if ( ! empty( $val ) ) {
				if ( 'all' === $val ) {
					self::history_cleanup();
				} elseif ( is_numeric( $val ) ) {
					global $wpdb;
					$wpdb->query( $wpdb->prepare(
						' DELETE from ' . SISANU_EMAILS_CATCH_ALL_TABLE . ' WHERE id = %d ',
						(int) $val
					) );
				}
			}
		}
	}

	/**
	 * The plugin settings and history page.
	 *
	 * @access public
	 * @static
	 *
	 * @return void
	 */
	public static function emails_catch_all_settings() {
		// Verify user capabilities in order to deny the access if the user does not have the capabilities.
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( esc_html( 'Action not allowed.', 'secas' ) );
		}

		$t2  = filter_input( INPUT_POST, 'submit2', FILTER_DEFAULT );
		$t3  = filter_input( INPUT_POST, 'submit3', FILTER_DEFAULT );

		$current_tab = 'secas-menu-item-1';
		if ( ! empty( $t2 ) ) {
			$current_tab = 'secas-menu-item-2';
		} elseif ( ! empty( $t3 ) ) {
			$current_tab = 'secas-menu-item-3';
		}

		self::get_options();
		?>
		<div class="wrap ic-devops-wrap">
			<h1><span class="dashicons dashicons-email ic-devops"></span><?php esc_html_e( 'Emails Catch All', 'secas' ); ?></h1>
			<br/>
			<div class="secas-wrapper">
				<form action="" method="post" autocomplete="off">
					<?php wp_nonce_field( '_secas_settings_save', '_secas_settings_nonce' ); ?>
					<ul class="secas-menu" data-active="<?php echo esc_attr( $current_tab ); ?>">
						<li id="secas-menu-item-1"><?php esc_html_e( 'Settings', 'secas' ); ?></li>
						<?php if ( false === self::$settings->just_record ) : ?>
							<li id="secas-menu-item-2"><?php esc_html_e( 'Catch All Options', 'secas' ); ?></li>
							<li id="secas-menu-item-3"><?php esc_html_e( 'SMTP Options', 'secas' ); ?></li>
						<?php endif; ?>
					</ul>
					<ul class="secas-menu-items">
						<li id="secas-menu-item-1-content">
							<label><input type="radio" name="_secas_settings[just_record]" id="_secas_settings_just_record_1" value="1" <?php checked( self::$settings->just_record, true ); ?>/> <?php esc_html_e( 'Just record the emails', 'secas' ); ?></label>, <label><input type="radio" name="_secas_settings[just_record]" id="_secas_settings_just_record_0" value="0" <?php checked( self::$settings->just_record, false ); ?>/> <?php esc_html_e( 'Use more options that allows also the replace or receive of emails', 'secas' ); ?></label>
							<?php submit_button( __( 'Update Settings', 'secas' ), 'primary', '', false ); ?>
						</li>
						<?php if ( false === self::$settings->just_record ) : ?>
							<li id="secas-menu-item-2-content">
								<table width="100%" cellpadding="5">
									<tr>
										<td colspan="3">
											<h3><?php esc_html_e( 'Catch All Options', 'secas' ); ?></h3>
											<hr>
										</td>
									</tr>
									<tr>
										<td><b><?php esc_html_e( 'Recipients', 'secas' ); ?></b></td>
										<td>
											<input type="text" name="_secas_settings[email]" value="<?php echo esc_attr( self::$settings->email ); ?>" size="40" />
										</td>
										<td>
											<?php esc_html_e( 'The email address/addresses (separated with comma) to be used to receive a copy/replace all email recipients sent from the application', 'secas' ); ?>
											<?php if ( empty( self::$settings->email ) ) : ?>
												<h2><?php esc_html_e( 'Please note that if the catch all email address is not set, the plugin catch all settings will not be applied.', 'secas' ); ?></h2>
											<?php endif; ?>
										</td>
									</tr>
									<tr>
										<td nowrap="nowrap"><b><?php esc_html_e( 'Recipient Type', 'secas' ); ?></b></td>
										<td>
											<label><input type="radio" name="_secas_settings[recipient]" id="_secas_settings_receive" value="receive" <?php checked( 'receive', self::$settings->recipient, true ); ?>/> <b><?php esc_html_e( 'RECEIVE A COPY', 'secas' ); ?></b></label>
										</td>
										<td>
											<?php esc_html_e( 'The emails will be sent to the intended recipients as normal + a copy of all emails will be sent also to the catch all email address specified above', 'secas' ); ?>
										</td>
									</tr>
									<tr>
										<td></td>
										<td>
											<label><input type="radio" name="_secas_settings[recipient]" id="_secas_settings_replace" value="replace" <?php checked( 'replace', self::$settings->recipient, true ); ?>/> <b><?php esc_html_e( 'REPLACE ALL RECIPIENTS', 'secas' ); ?></b></label>
										</td>
										<td>
											<?php esc_html_e( 'The emails will be sent only to the catch all email address specified above, the intended recipient will not receive the emails', 'secas' ); ?>
										</td>
									</tr>
									<tr>
										<td colspan="3"><hr></td>
									</tr>
									<tr>
										<td><b><?php esc_html_e( 'Content Type', 'secas' ); ?></b></td>
										<td>
											<label><input type="radio" name="_secas_settings[content_type]" id="_secas_settings_content_type_plain" value="plain" <?php checked( 'plain', self::$settings->content_type, true ); ?>> <?php esc_html_e( 'Plain Text', 'secas' ); ?></label>, <label><input type="radio" name="_secas_settings[content_type]" id="_secas_settings_content_type_html" value="html" <?php checked( 'html', self::$settings->content_type, true ); ?>><?php esc_html_e( 'HTML', 'secas' ); ?></label>
										</td>
										<td><?php esc_html_e( 'The content type for email sent through', 'secas' ); ?></td>
									</tr>
									<tr>
										<td colspan="3"><hr></td>
									</tr>
									<tr>
										<td nowrap="nowrap"><b><?php esc_html_e( 'Record Emails', 'secas' ); ?></b></td>
										<td>
											<label><input type="radio" name="_secas_settings[history]" id="_secas_settings_history_0" value="0" <?php checked( 0, self::$settings->history, true ); ?>><?php esc_html_e( 'No', 'secas' ); ?></label>, <label>
											<input type="radio" name="_secas_settings[history]" id="_secas_settings_history_1" value="1" <?php checked( 1, self::$settings->history, true ); ?>><?php esc_html_e( 'Yes', 'secas' ); ?></label>
										</td>
										<td><?php esc_html_e( 'Record the emails for later review', 'secas' ); ?></td>
									</tr>
									<?php if ( ! empty( self::$settings->email ) ) : ?>
										<tr>
											<td colspan="3"><hr></td>
										</tr>
										<tr>
											<td><b><?php esc_html_e( 'Test', 'secas' ); ?></b></td>
											<td>
												<label><input type="checkbox" name="_secas_settings[test]" id="_secas_settings_test" value=""><?php esc_html_e( 'Send a test email', 'secas' ); ?></label>
											</td>
											<td><?php esc_html_e( 'Send a test email to see the settings in action', 'secas' ); ?></td>
										</tr>
									<?php endif; ?>
								</table>
								<?php submit_button( __( 'Update Settings', 'secas' ), 'primary', 'submit2', true ); ?>
							</li>
						<?php endif; ?>

						<li id="secas-menu-item-3-content">
							<table width="100%" cellpadding="5">
								<tr>
									<td colspan="3">
										<h3><?php esc_html_e( 'SMTP Options', 'secas' ); ?></h3>
										<hr>
									</td>
								</tr>
								<tr>
									<td><b><?php esc_html_e( 'SMTP Auth', 'secas' ); ?></b></td>
									<td>
										<label><input type="radio" name="_secas_settings[smtp_auth]" id="_secas_settings_smtp_auth_1" value="1" <?php checked( true, self::$settings->smtp_auth, true ); ?>/> <?php esc_html_e( 'Yes', 'secas' ); ?></label>, <label><input type="radio" name="_secas_settings[smtp_auth]" id="_secas_settings_smtp_auth_0" value="0" <?php checked( false, self::$settings->smtp_auth, true ); ?>/> <?php esc_html_e( 'No', 'secas' ); ?></label>
									</td>
									<td><?php esc_html_e( 'Enable SMTP authentication', 'secas' ); ?></td>
								</tr>
								<tr>
									<td><b><?php esc_html_e( 'Host', 'secas' ); ?></b></td>
									<td><input type="text" name="_secas_settings[smtp_host]" value="<?php echo esc_attr( self::$settings->smtp_host ); ?>" size="40" /></td>
									<td><?php esc_html_e( 'Specify the SMTP servers (separate with ; if you have more, for example: smtp1.yourdomain.com;smtp2.yourdomain.com)', 'secas' ); ?></td>
								</tr>
								<tr>
									<td><b><?php esc_html_e( 'Port', 'secas' ); ?></b></td>
									<td><input type="text" name="_secas_settings[smtp_port]" value="<?php echo esc_attr( self::$settings->smtp_port ); ?>" size="10" /></td>
									<td><?php esc_html_e( 'The TCP port to connect to', 'secas' ); ?></td>
								</tr>
								<tr>
									<td><b><?php esc_html_e( 'Username', 'secas' ); ?></b></td>
									<td><input type="text" name="_secas_settings[smtp_uname]" value="<?php echo esc_attr( self::$settings->smtp_uname ); ?> " size="40" autocomplete="off"></td>
									<td><?php esc_html_e( 'SMTP username (for example user@yourdomain.com)', 'secas' ); ?></td>
								</tr>
								<tr>
									<td><b><?php esc_html_e( 'Password', 'secas' ); ?></b></td>
									<td><input type="password" name="_secas_settings[smtp_upass]" value="<?php echo esc_attr( self::$settings->smtp_upass ); ?>" size="40" autocomplete="off"></td>
									<td><?php esc_html_e( 'SMTP password', 'secas' ); ?></td>
								</tr>
								<tr>
									<td colspan="3"><hr></td>
								</tr>
								<tr>
									<td><b><?php esc_html_e( 'SMTP Secure', 'secas' ); ?></b></td>
									<td>
										<label><input type="radio" name="_secas_settings[smtp_secure]" id="_secas_settings_smtp_secure_tls" value="tls" <?php checked( 'tls', self::$settings->smtp_secure, true ); ?>/> <?php esc_html_e( 'TLS', 'secas' ); ?></label>, <label><input type="radio" name="_secas_settings[smtp_secure]" id="_secas_settings_smtp_secure_ssl" value="ssl" <?php checked( 'ssl', self::$settings->smtp_secure, true ); ?>/> <?php esc_html_e( 'SSL', 'secas' ); ?></label>, <label><input type="radio" name="_secas_settings[smtp_secure]" id="_secas_settings_smtp_secure_" value="" <?php checked( '', self::$settings->smtp_secure, true ); ?>/> <?php esc_html_e( 'N/A', 'secas' ); ?></label>
									</td>
									<td><?php esc_html_e( 'Enable TLS or SSL encryption (set N/A if you are not sure)', 'secas' ); ?></td>
								</tr>
								<tr>
									<td colspan="3"><hr></td>
								</tr>
								<tr>
									<td><b><?php esc_html_e( 'From Email', 'secas' ); ?></b></td>
									<td><input type="email" name="_secas_settings[smtp_from]" value="<?php echo esc_attr( self::$settings->smtp_from ); ?>" size="40" /></td>
									<td><?php esc_html_e( 'The email address to be used as from for the emails', 'secas' ); ?></td>
								</tr>
								<tr>
									<td><b><?php esc_html_e( 'From Name', 'secas' ); ?></b></td>
									<td><input type="text" name="_secas_settings[smtp_from_name]" value="<?php echo esc_attr( self::$settings->smtp_from_name ); ?>" size="40" /></td>
									<td><?php esc_html_e( 'The name to be used as from for the emails', 'secas' ); ?></td>
								</tr>
							</table>
							<?php submit_button( __( 'Update Settings', 'secas' ), 'primary', 'submit3', true ); ?>
						</li>
					</ul>
				</form>
				<?php if ( ! empty( self::$settings->history ) || ! empty( self::$settings->just_record ) ) : ?>
					<div class="secas-list">
						<?php self::load_emails_page(); ?>
					</div>
				<?php endif; ?>
			</div>
		</div>
		<?php
	}

	/**
	 * Fix the links like <http://....> that are generated in the recent WP core.
	 *
	 * @access public
	 * @static
	 *
	 * @param  string $content The email message.
	 * @return string
	 */
	public static function fix_new_links_format( $content ) {
		preg_match_all( '/<http(.*)>/', $content, $matches );
		if ( ! empty( $matches[0] ) && ! empty( $matches[1] ) ) {
			foreach ( $matches[0] as $key => $value ) {
				if ( ! empty( self::$settings->content_type ) && 'plain' === self::$settings->content_type ) {
					$content = str_replace( $value, 'http' . $matches[1][ $key ], $content );
				} else {
					$content = str_replace( $value, '<a href="http' . $matches[1][ $key ] . '">http' . $matches[1][ $key ] . '</a>', $content );
				}
			}
		}
		$content = nl2br( $content );
		return $content;
	}

	/**
	 * Normalize the headers
	 *
	 * @param string|array $headers Email headers.
	 * @return array
	 */
	public static function prepare_headers( $headers ) {
		// Fix the headers, to convert all to array.
		if ( ! empty( $headers ) ) {
			if ( ! is_array( $headers ) ) {
				// Explode the headers out, so this function can take both string headers and an array of headers.
				$headers = explode( "\n", str_replace( "\r\n", "\n", $headers ) );
			} else {
				$headers = $headers;
			}
		} else {
			$headers = array();
		}
		return $headers;
	}

	/**
	 * Normalize the headers
	 *
	 * @param string|array $headers Email headers.
	 * @return array
	 */
	public static function analyze_headers( $headers ) {
		// Fix the headers, to convert all to array.
		$all_headers = array(
			'from' => array(),
			'to'   => array(),
			'cc'   => array(),
			'bcc'  => array(),
			'any'  => array(),
		);
		if ( ! empty( $headers ) ) {
			foreach ( $headers as $header ) {
				$lower = strtolower( $header );
				if ( 'bcc: ' === substr( $lower, 0, 5 ) ) {
					$all_headers['bcc'][] = $header;
				} elseif ( 'cc: ' === substr( $lower, 0, 4 ) ) {
					$all_headers['cc'][] = $header;
				} elseif ( 'from: ' === substr( $lower, 0, 6 ) ) {
					$all_headers['from'][] = $header;
				} elseif ( 'to: ' === substr( $lower, 0, 4 ) ) {
					$all_headers['to'][] = $header;
				} else {
					$all_headers['any'][] = $header;
				}
			}
		}
		return $all_headers;
	}

	/**
	 * Pack all headers into one array.
	 *
	 * @param array $headers Email headers.
	 * @return array
	 */
	public static function pack_headers( $headers ) {
		// Pack all headers into one array.
		$all_headers = array();
		if ( ! empty( $headers ) ) {
			foreach ( $headers as $header ) {
				if ( ! empty( $header ) ) {
					if ( is_array( $header ) ) {
						$all_headers = array_merge( $all_headers, $header );
					} else {
						$all_headers[] = $header;
					}
				}
			}
			// Make sure we remove empty value.
			$all_headers = array_filter( $all_headers );
		}
		return $all_headers;
	}

	/**
	 * Update the native mail headers and record history if the case.
	 *
	 * @access public
	 * @static
	 *
	 * @param  array $args Email arguments.
	 * @return array
	 */
	public static function wp_mail_catch_all( $args = array() ) {
		$new_wp_mail            = $args;
		$new_wp_mail['headers'] = self::prepare_headers( $new_wp_mail['headers'] );
		$computed               = self::analyze_headers( $new_wp_mail['headers'] );
		$initial_computed       = $computed;
		$bcc                    = '';
		if ( ! empty( self::$settings->email ) ) {
			if ( 'receive' === self::$settings->recipient ) {
				array_push( $computed['bcc'], 'Bcc: ' . self::$settings->email );
				$bcc = self::$settings->email;
			} elseif ( 'replace' === self::$settings->recipient ) {
				$new_wp_mail['to'] = self::$settings->email;
				unset( $computed['to'] );
				unset( $computed['cc'] );
				unset( $computed['bcc'] );
			}
		}
		$new_wp_mail['headers'] = self::pack_headers( $computed );
		$new_wp_mail['message'] = self::fix_new_links_format( $new_wp_mail['message'] );

		if ( ! empty( self::$settings->history ) || ! empty( self::$settings->just_record ) ) {
			$date  = current_time( 'timestamp' );
			$final = $new_wp_mail;
			if ( ! empty( self::$settings->content_type ) && 'plain' === self::$settings->content_type ) {
				$final['message'] = strip_tags( $final['message'] );
			}
			$final['bcc'] = '';
			if ( ! empty( $bcc ) ) {
				$final['bcc'] = $bcc;
			}

			$args['to']  = ( is_array( $args['to'] ) ) ? implode( ',', $args['to'] ) : $args['to'];
			$args['cc']  = ( ! empty( $initial_computed['cc'] ) ) ? implode( ',', $initial_computed['cc'] ) : '';
			$args['bcc'] = ( ! empty( $initial_computed['bcc'] ) ) ? implode( ',', $initial_computed['bcc'] ) : '';
			$final['to'] = ( is_array( $final['to'] ) ) ? implode( ',', $final['to'] ) : $final['to'];
			$opt         = array(
				'date'    => $date,
				'initial' => $args,
				'final'   => $final,
				'type'    => self::$settings->recipient,
			);

			if ( ! empty( $final['message'] ) ) {
				global $wpdb;
				$arr  = array(
					'date'    => $opt['date'],
					'type'    => $opt['type'],
					'initial' => maybe_serialize( $args['to'] ),
					'final'   => maybe_serialize( $final['to'] ),
					'subject' => $final['subject'],
					'content' => ( ! empty( $final['message'] ) ) ? self::fix_new_links_format( $final['message'] ) : '',
					'all'     => maybe_serialize( $opt ),
				);
				$type = array( '%d', '%s', '%s', '%s', '%s', '%s', '%s' );
				$wpdb->insert( SISANU_EMAILS_CATCH_ALL_TABLE, $arr, $type );
			}
		}

		return $new_wp_mail;
	}

	/**
	 * The actions to be executed when the plugin is deactivated.
	 *
	 * @access public
	 * @static
	 *
	 * @return void
	 */
	public static function deactivate_plugin() {
		self::history_cleanup();
		delete_option( self::$option_name );
		delete_option( self::$option_name . '_db_version' );
		global $wpdb;
		$wpdb->query( 'DROP TABLE IF EXISTS ' . SISANU_EMAILS_CATCH_ALL_TABLE );
	}

	/**
	 * The actions to be executed when the plugin is activated.
	 *
	 * @access public
	 * @static
	 *
	 * @return void
	 */
	public static function activate_plugin() {
		global $wpdb;
		$db_version = get_option( self::$option_name . '_db_version', 0 );
		if ( SISANU_EMAILS_CATCH_ALL_DB_VERSION !== (float) $db_version ) {
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			$wpdb->query( 'DROP TABLE IF EXISTS ' . SISANU_EMAILS_CATCH_ALL_TABLE );
			$sql = ' CREATE TABLE ' . SISANU_EMAILS_CATCH_ALL_TABLE . ' ( `id` BIGINT(20) AUTO_INCREMENT, `date` BIGINT(20), `type` VARCHAR(15), `initial` VARCHAR(255), `final` VARCHAR(255), `subject` VARCHAR(255), `content` TEXT, `all` TEXT, UNIQUE KEY `id` (id) ) CHARACTER SET utf8 COLLATE utf8_general_ci COMMENT = \'Table created by Emails Catch All Plugin\'';
			dbDelta( $sql );
			update_option( self::$option_name . '_db_version', SISANU_EMAILS_CATCH_ALL_DB_VERSION );
		}
	}

	/**
	 * Add the settings and the plugin link.
	 *
	 * @access public
	 * @static
	 *
	 * @param  array $links Plugin links array.
	 * @return array
	 */
	public static function plugin_action_links( $links ) {
		$all   = array();
		$all[] = '<a href="' . esc_url( admin_url( self::$plugin_url ) ) . '">' . __( 'Settings', 'secas' ) . '</a>';
		$all[] = '<a href="https://iuliacazan.ro/emails-catch-all">' . __( 'Plugin URL', 'secas' ) . '</a>';
		$all   = array_merge( $all, $links );
		return $all;
	}
}

$sisanu_emails_catch_all = SISANU_Emails_Catch_All::get_instance();
register_activation_hook( __FILE__, array( $sisanu_emails_catch_all, 'activate_plugin' ) );
register_deactivation_hook( __FILE__, array( $sisanu_emails_catch_all, 'deactivate_plugin' ) );
